//
// BitStreaming API sample code.
// Sample program for use with bitstrmwdm.sys device driver.
// Copyright (c) 1999 Sangoma Technologies Inc.
// Use at your own risk.
// Target: Win32 Console
//

#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <conio.h>

#include "bstrmapi.h"
#include "sangbitstrm.h"


HANDLE drv;		//handle for commands

BSTRM_MAILBOX_STRUCT * pcb;

#define RX_BUFF_LEN sizeof(BSTRM_DATA_RX_STATUS_EL_STRUCT) + PRI_MAX_LENGTH_TX_DATA_BLOCK

//buffer used for receiving data
UCHAR rx_data[RX_BUFF_LEN];

//////////////////////////////////////////////////
//structures and definitions used for sending data
//
typedef struct
{
	void * previous;
	TX_DATA_STRUCT tx_data;
}tx_queue_element_t;

typedef struct
{
	//number of nodes in the list
	USHORT size;
	//insert at tail
	tx_queue_element_t * tail;
	//remove from head
	tx_queue_element_t * head;
	//mutex for synchronizing access to the queue
	HANDLE tx_queue_mutex;
}tx_queue_t;

//queue for transmitting data
tx_queue_t tx_queue;

#define TX_MUTEX_TIMEOUT	1000//1 second
#define MAX_TX_QUEUE_SIZE	500	//optimal length
//
//end of structures and definitions used for sending data
//////////////////////////////////////////////////

HANDLE hTxWaitHandle;

UCHAR cbdata[6000];

ULONG data_blocks_count=0;

char c;

BOOLEAN stop_tx = FALSE;

//
//Additional initialization should be done
//for S5144 and S5147 T1E1 cards.
//
//#define BUILD_FOR_T1E1_CARD

#ifdef BUILD_FOR_T1E1_CARD
	//define only one of these!
//	#define BUILD_T1
	#define BUILD_E1
#endif

#if defined (BUILD_T1) && defined (BUILD_E1)
#error "Can not build for both T1 and E1!"
#endif

#if defined(BUILD_E1)
	#define TX_HDLC_LEN	E1_TX_LENGTH	//must be divisible by 31
#else
	#define TX_HDLC_LEN	T1_TX_RX_LENGTH
#endif

UCHAR tx_buff[TX_HDLC_LEN];

char * device_name = "\\\\.\\SANGBITSTRM0";	//Windows 2000/XP, S508 and S514

UCHAR 
DoCommand(
	HANDLE drv, 
	BSTRM_MAILBOX_STRUCT * pcb
	);

UCHAR 
DoWriteCommand(
	HANDLE drv, 
	TX_DATA_STRUCT * pTx
	);

USHORT
DoReadCommand(
	HANDLE drv, 
	PUCHAR dat, 
	ULONG maxl
	);

DWORD 
RxThreadFunc(
	LPDWORD lpdwParam
	);

DWORD 
TxThreadFunc(
	LPDWORD lpdwParam
	);

VOID 
ConfigureTECardCommand(
	HANDLE drv, 
	TE_CFG_T * te_cfg
	);

VOID 
TE1MonitorCommand(
	HANDLE drv, 
	BSTRM_MAILBOX_STRUCT * mb
	);

VOID 
GetMemoryDump(
	HANDLE drv, 
	PMEMORY_DUMP pMem_dump
	);

VOID 
GetApiStatistics(
	HANDLE drv, 
	PAPI_STATS pApi_stats
	);

int set_bstrm_configuration();

int set_te1_configuration();

void main_loop();

int tx_enqueue(unsigned char * buffer, unsigned int length);
int tx_dequeue(TX_DATA_STRUCT * destination);
void empty_tx_queue();

void main()
{
    DWORD dwThreadId, dwThrdParam = 1; 
    HANDLE hRxThread, hTxThread; 
	int i;
	
	pcb = (BSTRM_MAILBOX_STRUCT *)cbdata;

	// Open device number 0
	drv = CreateFile(device_name,
				GENERIC_READ | GENERIC_WRITE,
				FILE_SHARE_READ | FILE_SHARE_WRITE,
				(LPSECURITY_ATTRIBUTES)NULL,
				OPEN_EXISTING,
				FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, 
				(HANDLE)NULL);
    if (drv == INVALID_HANDLE_VALUE)
	{	printf("Unable to open %s!\n", device_name);
		return;
	}

#ifdef BUILD_FOR_T1E1_CARD

	if(set_te1_configuration() == FALSE)
		return;

#endif//#ifdef BUILD_FOR_T1E1_CARD

	if(set_bstrm_configuration() == FALSE)
		return;

	// enable the transmitter and receiver (check the return_code).
	printf("Performing ENABLE_BSTRM_COMMUNICATIONS\n");

 	pcb->command = ENABLE_BSTRM_COMMUNICATIONS;
	pcb->buffer_length = 0;
	DoCommand(drv, pcb);

	printf("Rcode = 0x%02X, blength = %u\n", pcb->return_code, pcb->buffer_length);

	//any other non-zero return code is an error
	if(pcb->return_code)
	{	// close the open handle
		CloseHandle(drv);
		return;
	}

	//create Tx wait object
	hTxWaitHandle = CreateEvent( NULL, FALSE, FALSE, NULL);
	if(hTxWaitHandle == NULL)
	{	printf("Failed to create Tx event!!\n");
		// close the open handle
		CloseHandle(drv);
		return;
	}

	//initialize tx queue.
	memset(&tx_queue, 0x00, sizeof(tx_queue_t));
	tx_queue.tx_queue_mutex = CreateMutex(NULL, FALSE, NULL);
	if(tx_queue.tx_queue_mutex == NULL){
		printf("Failed to create tx queue mutex!!!!\n");
		return;
	}

	//start a thread for writing only
	hTxThread = CreateThread( 
		NULL,									// no security attributes
		0,										// use default stack size
		(LPTHREAD_START_ROUTINE) TxThreadFunc,	// thread function
		&dwThrdParam,							// argument to thread function
		0,										// use default creation flags
		&dwThreadId								// returns the thread identifier
		);

	if(hTxThread == NULL){
		printf("failed to create tx thread!!\n");
		return;
	}

	if(SetThreadPriority(hTxThread, THREAD_PRIORITY_TIME_CRITICAL) == FALSE){
		printf("failed to set tx thread priority!!\n");
		return;
	}

	// start a thread for reading only
    hRxThread = CreateThread( 
        NULL,									// no security attributes        
        0,										// use default stack size         
        (LPTHREAD_START_ROUTINE) RxThreadFunc,	// thread function        
        &dwThrdParam,							// argument to thread function    
        0,										// use default creation flags     
        &dwThreadId);							// returns the thread identifier  

	if(hRxThread == NULL){
		printf("failed to create rx thread!!\n");
		return;
	}

	if(SetThreadPriority(hRxThread, THREAD_PRIORITY_TIME_CRITICAL) == FALSE){
		printf("failed to set rx thread priority!!\n");
		return;
	}

	//wait for a short time, so messages will be printed in order.
	Sleep(10);

	//enter main user interaction loop
	main_loop();

	//main_loop() returned, it means user wants to exit the program.
	printf("\nPress any key to disable comms.\n");
	getch();

	//
	//turn off receiver and transmitter - this also
	//cancels any pending read and write commands
	//
	i=0;
	do {
		pcb->command = DISABLE_BSTRM_COMMUNICATIONS;
		pcb->buffer_length = 0;
		DoCommand(drv, pcb);
		printf("Disable comms rcode = 0x%02X.\n", pcb->return_code);
	} while (pcb->return_code && i++ < 5);

	printf("WRITE:Press any key to close device...\n");
	getch();

	// close the open handle
	CloseHandle(drv);
}

// perform a standard command
UCHAR DoCommand(HANDLE drv, BSTRM_MAILBOX_STRUCT * pcb)
{
	DWORD ln;

	if (DeviceIoControl(
		drv,
		IoctlCodeCblockCommand,
		(LPVOID)pcb,
		(ULONG)pcb->buffer_length + 16L,
		(LPVOID)pcb,
		sizeof(cbdata),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		) == FALSE)
		pcb->return_code = 0x7d;		// STATUS_UNSUCCESSFUL
	return(pcb->return_code);
}

//
// To transmit a block of data do blocking write command.
// On return check the 'operation_status'.
// 
UCHAR 
DoWriteCommand(
	HANDLE drv, 
	TX_DATA_STRUCT * pTx
	)
{
	DWORD ln;

	if(DeviceIoControl(
			drv,
			IoctlCodeWriteCommand,
			(LPVOID)pTx,
			sizeof(TX_DATA_STRUCT),
			(LPVOID)pTx,
			sizeof(TX_DATA_STRUCT),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
			) == FALSE
	   )
		return (0x7d);		// STATUS_UNSUCCESSFUL
	else
		return (0);
}

// do blocking read command
// received data block is stored in user buffer on return
USHORT DoReadCommand(HANDLE drv, PUCHAR dat, ULONG maxl)
{
	DWORD ln;
	USHORT rc=0;

	if (DeviceIoControl(
		drv,
		IoctlCodeReadCommand,
		(LPVOID)NULL,
		0L,
		(LPVOID)dat,
		maxl,
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		) == FALSE)
		rc = 1;				// STATUS_UNSUCCESSFUL

	return rc;
}

VOID 
ConfigureTECardCommand(
	HANDLE drv, 
	TE_CFG_T * te_cfg
	)
{
	DWORD ln;

	DeviceIoControl(
		drv,
		IoctlSetTEConfiguration,
		(LPVOID)te_cfg,
		sizeof(TE_CFG_T),
		(LPVOID)te_cfg,
		sizeof(TE_CFG_T),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}

VOID 
TE1MonitorCommand(
	HANDLE drv, 
	BSTRM_MAILBOX_STRUCT * mb
	)
{
	DWORD ln;

	DeviceIoControl(
		drv,
		Ioctl_TE_56K_MonitorCommand,
		(LPVOID)mb,
		sizeof(BSTRM_MAILBOX_STRUCT),
		(LPVOID)mb,
		sizeof(BSTRM_MAILBOX_STRUCT),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}

VOID 
GetMemoryDump(
	HANDLE drv, 
	PMEMORY_DUMP pMem_dump
	)
{
	DWORD ln;

	DeviceIoControl(
		drv,
		Ioctl_GET_MEM_DUMP,
		(LPVOID)pMem_dump,
		sizeof(MEMORY_DUMP),
		(LPVOID)pMem_dump,
		sizeof(MEMORY_DUMP),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}

// read thread
DWORD RxThreadFunc(LPDWORD lpdwParam)
{ 
	USHORT					datlen, j, jn;
	PSTREAMING_READ_INFO	pri;
	HANDLE					rx_drv;	//handle dedicated to receiving only
	static int				print_counter = 0;

	static unsigned char tx_data_buffer[PRI_MAX_LENGTH_TX_DATA_BLOCK];

    printf("\nRxThreadFunc:  thread parameter=%d\n", *lpdwParam); 

	// open another handle strictly for reading
	rx_drv = CreateFile(device_name, 
						GENERIC_READ | GENERIC_WRITE,
						FILE_SHARE_READ | FILE_SHARE_WRITE,
						(LPSECURITY_ATTRIBUTES)NULL,
						OPEN_EXISTING,
						FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH,
						(HANDLE)NULL
						);

    if (rx_drv == INVALID_HANDLE_VALUE)
	{	printf("\nUnable to open %s for reading!\n", device_name);
		return(0);
	}

	printf("RX_BUFF_LEN : %d\n", RX_BUFF_LEN);

	while (DoReadCommand(rx_drv, rx_data, RX_BUFF_LEN) == 0)
	{	//printf("\nNonBuffered mode:Waiting to read something...\n");
		//use our special buffer at rx_data to hold received block of data.
		pri = (PSTREAMING_READ_INFO)rx_data;
		
		if(pri->Status == RX_DATA_TIMEOUT)
		{	//no data in 2 seconds, try again.
			printf("Timeout on read.\n");
			continue;
		}else if(pri->Status == RX_DATA_AVAILABLE)
		{
			if(pri->MiscBits != 0)
			{	//Corresponds to 'error_flag' in BSTRM_DATA_RX_STATUS_EL_STRUCT
				//Discard this block of data.
				printf("MBits=0x%02X\n", pri->MiscBits);
				continue;
			}
			
			// a block of data was received - dump it without first 16 header bytes
			datlen = pri->FrameLength;
			data_blocks_count++;

			//print every 200 data blocks
			if(print_counter && (print_counter++ % 200) == 0){
				print_counter = 0;

				printf("Rx len %u\n", datlen);
				//This will print out ALL the data, can be too much to print!
				//Printing to screen is very slow process which may cause
				//loss of Rx data.
				for(j = 0; j < datlen; j++)
				{	printf("%02X ", pri->Data[j]);
				}
				printf("\n");
			}

			///////////////////////////////////////////////////////////////////////
			//This will send the received Bit Stream back to the sender - loopback.
			//
#if defined(BUILD_E1)
			//on E1 discard Framing byte at channel 0.
			jn = 0;
			for(j = 0; j < datlen; j++){
				if((j%32)!=0){
					//tx_data.frame_data[jn] = pri->Data[j];
					tx_data_buffer[jn] = pri->Data[j];
					jn++;
				}
			}
			//queue for transmission
			tx_enqueue(tx_data_buffer, jn);
#else
			memcpy(tx_data_buffer, pri->Data, datlen);
			//queue for transmission
			tx_enqueue(tx_data_buffer, datlen);
#endif

			if(hTxWaitHandle != NULL)
			{	//signal to tx thread to send a block of data
				SetEvent(hTxWaitHandle);
			}
			//
			//End of loop back code
			///////////////////////////////////////////////////////////////////////
		}
	}//while()
	
	//The DoReadCommand() may fail for the following reasons:
	//
	//1. DISABLE_BSTRM_COMMUNICATIONS was done
	//2. The previous DoReadCommand() is not complete yet
	//	 ( some other thread or proccess is trying to
	//	 receive data from this device).
	printf("\nREAD Thread : closing the driver...\n");
	CloseHandle(rx_drv);
	printf("READ Thread : terminating...\n");
    return 0; 
}
 
// Write Thread
DWORD 
TxThreadFunc(
	LPDWORD lpdwParam
	)
{ 
	HANDLE	drv_rx;
	DWORD	dwWaitResult;

	TX_DATA_STRUCT tx_data;

    printf("\nTxThreadFunc:  thread parameter=%d\n", *lpdwParam); 
	
	//open another handle strictly for writing
	drv_rx = CreateFile(
		device_name, 
		GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		(LPSECURITY_ATTRIBUTES)NULL,
		OPEN_EXISTING,
		FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH,
		(HANDLE)NULL
		);
	
    if (drv_rx == INVALID_HANDLE_VALUE)
	{	printf("\nUnable to open %s for writing!\n", device_name);
		return(0);
	}
	
	//start the transmission
	while(c != 'q' && stop_tx == FALSE)
	{	
		//wait for tx event to be set to signaled state.
		dwWaitResult = WaitForSingleObject(hTxWaitHandle, 1000);
		
		switch(dwWaitResult)
		{
		case WAIT_TIMEOUT:
			continue;
		case WAIT_ABANDONED:
			printf("Tx WAIT_ABANDONED\n.");
			stop_tx = TRUE;
			break;
		}

		if(stop_tx == TRUE)
			break;

		while(tx_dequeue(&tx_data) == 0)
		{	//
			//The tx_data structure MUST be initialized before calling DoWriteCommand()!
			//
			if(	DoWriteCommand(drv_rx, &tx_data))
			{	printf("Communications disabled or previouse DoWriteCommand()\
has not completed yet!\n");
				stop_tx = TRUE;
				break;
			}

			//check that block of data was transmitted
			switch(tx_data.operation_status)
			{
			case TX_OK:
				//printf("Block of data queued for transmission.\n");
				break;
						
			case TX_TIMEOUT:
				//it means that tx buffers are constanly full,
				//usually because the line is disconnected
				printf("****** TX_TIMEOUT ******\n");
				//stop_tx = TRUE;
				break;
						
			case TX_RETRY:
				//a block of data was not transmitted !! should be retransmitted.
				printf("Retry Tx!\n");
				break;
						
			case TX_DATA_TOO_LONG:
				//maximum length is PRI_MAX_LENGTH_TX_DATA_BLOCK bytes
				printf("****** TX_DATA_TOO_LONG ******\n");
				break;
						
			case TX_DATA_TOO_SHORT:
				//minimum is 1 byte
				printf("****** TX_DATA_TOO_SHORT ******\n");
				break;

			default:
				printf("Unknown return code (0x%X) on transmission!\n",
					tx_data.operation_status);
				stop_tx = TRUE;
				break;
			}
		}//while()
	}//while()

	//The DoWriteCommand() may fail for the following reasons:
	//
	//1. DISABLE_BSTRM_COMMUNICATIONS was done
	//2. The previous DoWriteCommand() is not complete yet
	//	 ( some other thread or proccess is trying to
	//	 send data using this device).

	CloseHandle(hTxWaitHandle);

	printf("\nWrite Thread is closing the driver...\n");
	CloseHandle(drv_rx);
	printf("Write Thread is terminating...\n");

	empty_tx_queue();

    return 0; 
}
	
void main_loop()
{
	BSTRM_CONFIGURATION_STRUCT * strc;

	// Loop around waiting for user to request the transmission of the 
	// sample HDLC frame.
	// Or to execute user commands.
	do{
		printf("\nPress 'q' to quit or 'f' to get data_blocks_count or\n");
		printf("\t 'r' to read BitStream configuration or\n");
		printf("\t 'v' to read BitStream firmware version or\n");
		printf("\t 'u' to read number of transmit underruns or\n");
		printf("\t 's' to send sample HDLC frame.\n");

		c = tolower(getch());

		printf("\n");

		switch(c)
		{
		case 'f':
			printf("data_blocks_count:%ld\n", data_blocks_count);
			break;

		case 'r':
			strc = (BSTRM_CONFIGURATION_STRUCT *)pcb->data;
			
			pcb->command = READ_BSTRM_CONFIGURATION;
			pcb->buffer_length = 0;
			pcb->return_code = 0xaa;
			DoCommand(drv, pcb);
			printf("READ_BSTRM_CONFIGURATION rcode = 0x%02X.\n", pcb->return_code);

			if(pcb->return_code == 0)
			{	printf("baud_rate            : 0x%X\n", strc->baud_rate);
				printf("line_config_options  : 0x%X\n", strc->line_config_options);
				printf("modem_config_options : 0x%X\n", strc->modem_config_options);
				printf("modem_status_timer   : 0x%X\n", strc->modem_status_timer);
				printf("API_options          : 0x%X\n", strc->API_options);
				printf("SYNC_options         : 0x%X\n", strc->SYNC_options);
				printf("Rx_sync_char         : 0x%X\n", strc->Rx_sync_char);
				printf("monosync_Tx_time_fill_char : 0x%X\n", strc->monosync_Tx_time_fill_char);
				printf("buffer_options           : 0x%X\n", strc->buffer_options);
				printf("max_length_Tx_data_block : 0x%X\n", strc->max_length_Tx_data_block);
				printf("Rx_complete_length       : 0x%X\n", strc->Rx_complete_length);
				printf("Rx_complete_timer        : 0x%X\n", strc->Rx_complete_timer);;
				printf("statistics_options       : 0x%X\n", strc->statistics_options);
				printf("ptr_shared_mem_info_struct : 0x%X\n", strc->ptr_shared_mem_info_struct);
				printf("ptr_Tx_stat_el_cfg_struct  : 0x%X\n", strc->ptr_Tx_stat_el_cfg_struct);
				printf("ptr_Rx_stat_el_cfg_struct  : 0x%X\n", strc->ptr_Rx_stat_el_cfg_struct);

			}else
			{	printf("READ_BSTRM_CONFIGURATION failed!!!\n");
			}
			break;

		case 's':
			//set to contain HDLC flags
			memset(tx_buff, 0x7E, TX_HDLC_LEN);
			//Send a HDLC frame (THIS frame does not cause bitstuffing).
			// Data : 0x02
			// CRC	: 0x6AD3
			tx_buff[0x05] = 0x02;
			tx_buff[0x06] = 0x6A;
			tx_buff[0x07] = 0xD3;

			//insert data in to the tx queue
			tx_enqueue(tx_buff, TX_HDLC_LEN);

			if(hTxWaitHandle != NULL)
			{	//signal to tx thread to send a block of data
				SetEvent(hTxWaitHandle);
			}
			break;

		case 'v':
			pcb->command = READ_BSTRM_CODE_VERSION;
			pcb->buffer_length = 0;
			pcb->return_code = 0xaa;
			DoCommand(drv, pcb);
			printf("READ_BSTRM_CODE_VERSION rcode = 0x%02X.\n", pcb->return_code);

			if(pcb->return_code == 0)
			{	pcb->data[pcb->buffer_length] = '\0';
				printf("Code Version: %s\n", pcb->data);
			}
			break;

		case 'u':
			//Number of transmit underruns:
			//number of times the API transmitted buffer filled with
			//the 'monosync_Tx_time_fill_char' instead of user data.
			//If this counter is constantly increasing, it means that
			//the API user is not transmitting fast enough.
			{
				API_STATS api_stats;

				api_stats.return_code = 0xAA;

				GetApiStatistics(drv, &api_stats);

				if(api_stats.return_code == 0){
					printf("time_fill_char_tx_counter: %lu\n", 
						api_stats.time_fill_char_tx_counter);
				}else{
					printf("GetApiStatistics() failed!!\n");
				}

			}
			break;
		}
	} while (c != 'q');
}

int set_bstrm_configuration()
{
	BSTRM_CONFIGURATION_STRUCT * strc;
	unsigned int ln;
#if !defined( BUILD_FOR_T1E1_CARD )
	char c;
#endif

	// clear the configuration data area
	memset(cbdata, 0, sizeof(cbdata));

    // set up for a global configuration
	strc = (BSTRM_CONFIGURATION_STRUCT *)pcb->data;

	pcb->command = SET_BSTRM_CONFIGURATION;
	pcb->buffer_length = sizeof(BSTRM_CONFIGURATION_STRUCT);

	strc->modem_config_options = 0;
	strc->modem_status_timer = 500;
	strc->API_options = 0x00;	

	strc->buffer_options = 0;
	strc->statistics_options = 0;

#if !defined( BUILD_FOR_T1E1_CARD )

	printf("\nClock: 0. External, 1. 9600 bps 2. 64000 bps. 3.2750000 bps.? ");
	c = getche();

	switch(c)
	{
	case '0':
		strc->baud_rate = 0;
		break;
	case '1':
		strc->baud_rate = 9600;
		break;
	case '2':
		strc->baud_rate = 64000;
		break;
	case '3':
		strc->baud_rate = PRI_MAX_BAUD_RATE_S514;
		break;
	default:
		strc->baud_rate = 0;	// External clock
	}

	printf("\nConfigure for 'r's-232 or 'v'.35? ");

	if (getche() == 'r')
		strc->line_config_options = INTERFACE_LEVEL_RS232;
	else
		strc->line_config_options = INTERFACE_LEVEL_V35;
	printf("\n");

	strc->SYNC_options = MONOSYNC_8_BIT_SYNC;
	strc->Rx_sync_char = 0x7E;
	strc->monosync_Tx_time_fill_char = 0x7E;

	strc->max_length_Tx_data_block = PRI_MAX_LENGTH_TX_DATA_BLOCK;

	//If baud rate is low set 'Rx_complete_length' to a small value like 256,
	//so read command will return faster.
	strc->Rx_complete_length = PRI_MAX_LENGTH_TX_DATA_BLOCK;
	strc->Rx_complete_timer = 0;

#else

	strc->SYNC_options = EXTERNAL_SYNC_8_BIT_TIME_FILL;
	strc->Rx_sync_char = 0x00;
	strc->monosync_Tx_time_fill_char = 0x00;

#if defined(BUILD_T1)
	//must be divisibe by 24 for T1 line, by 32 for E1 line
	strc->max_length_Tx_data_block = T1_TX_RX_LENGTH;//720

	//must be divisibe by 24 for T1 line, by 32 for E1 line
	strc->Rx_complete_length = T1_TX_RX_LENGTH;//720
#elif defined(BUILD_E1)
	//must be divisibe by 24 for T1 line, by 31 for E1 line
	strc->max_length_Tx_data_block = E1_TX_LENGTH;	//992

	//must be divisibe by 24 for T1 line, by 32 for E1 line
	strc->Rx_complete_length = E1_RX_LENGTH;		//1024
#endif

	strc->Rx_complete_timer = 1500;

#endif

	printf("Performing SET_BSTRM_CONFIGURATION\n");
	// perform the command
	DoCommand(drv, pcb);

	// if bad return_code don't continue, find what's wrong.
	printf("Rcode = 0x%02X, blength = %u\n", pcb->return_code, pcb->buffer_length);
	for (ln=0; ln < (DWORD)pcb->buffer_length; ln++)
		printf("0x%02X ", (UCHAR)pcb->data[ln]);
	printf("\n");

	//any non-zero return code is an error
	if(pcb->return_code)
	{	//close the open handle
		CloseHandle(drv);
		return FALSE;
	}

	return TRUE;
}

int set_te1_configuration()
{
	TE_CFG_T te_cfg;

	// clear the TE1 configuration structure
	memset(&te_cfg, 0x00, sizeof(te_cfg));

#if defined (BUILD_T1)
	//set T1 configuration	
	te_cfg.media	= WANOPT_MEDIA_T1;
	te_cfg.lcode	= WANOPT_LC_B8ZS;
	te_cfg.frame	= WANOPT_FR_ESF;
	te_cfg.lbo		= WANOPT_T1_LBO_0_DB;
	te_cfg.te_clock	= WANOPT_NORMAL_CLK;
	//
	//A bitmap of channels that will be used by the API.
	//For example:
	//te_cfg.active_ch= 0xFF;//enable channels : 1 to 8
	//
	te_cfg.active_ch= TE_ENABLE_ALL_CHANNELS;
#endif

#if defined (BUILD_E1)
	//set E1 configuration	
	te_cfg.media	= WANOPT_MEDIA_E1;
	te_cfg.lcode	= WANOPT_LC_HDB3;
	te_cfg.frame	= WANOPT_FR_NCRC4;
	te_cfg.lbo		= WANOPT_T1_LBO_0_DB;
	te_cfg.te_clock	= WANOPT_NORMAL_CLK;
	te_cfg.active_ch= TE_ENABLE_ALL_CHANNELS;
#endif

	ConfigureTECardCommand(
		drv, 
		&te_cfg
		);

	printf("ConfigureTECardCommand return_code : ");

	switch(te_cfg.return_code)
	{
	case COMMAND_PERFORMED_SUCCESSFULLY:
		printf("COMMAND_PERFORMED_SUCCESSFULLY\n");
		break;

	case DO_DISABLE_BSTRM_COMMUNICATIONS:
		printf("DO_DISABLE_CHDLC_COMMUNICATIONS\n");
		break;

	case TE_CONFIGURATION_FAILED:
		//look at bitstrmlog.txt file in \WINNT\System32\drivers
		printf("TE_CONFIGURATION_FAILED\n");
		break;
	}

	//clean up!!
	if(te_cfg.return_code != COMMAND_PERFORMED_SUCCESSFULLY)
	{	// close the open handle
		CloseHandle(drv);
		return FALSE;
	}

	return TRUE;
}


int tx_enqueue(unsigned char * buffer, unsigned int length)
{
	tx_queue_element_t * element;
	DWORD mresult;

	mresult = WaitForSingleObject(tx_queue.tx_queue_mutex, TX_MUTEX_TIMEOUT);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED) {
		printf("tx_enqueue(): failed to get tx mutex!!!\n");		
		return 1;
	}

	if(tx_queue.size == MAX_TX_QUEUE_SIZE){
		printf("tx queue is full!\n");
		ReleaseMutex(tx_queue.tx_queue_mutex);
		return 2;
	}

	element = malloc(sizeof(tx_queue_element_t));
	if(element == NULL){
		printf("failed allocate memory for tx queue element!\n");
		ReleaseMutex(tx_queue.tx_queue_mutex);
		return 3;
	}

	//now copy everything in to the element
	memcpy(element->tx_data.frame_data, buffer, length);
	
	element->tx_data.frame_length = length;
	element->tx_data.operation_status = TX_TIMEOUT;

	//insert element at the tail of the queue
	element->previous = NULL;

	if(tx_queue.size == 0){
		//special case of a previously empty queue
		tx_queue.head = element;
		tx_queue.tail = element;
	}else{
		tx_queue.tail->previous = element;
		tx_queue.tail = element;
	}

	tx_queue.size++;

	ReleaseMutex(tx_queue.tx_queue_mutex);

	return 0;
}

int tx_dequeue(TX_DATA_STRUCT * destination)
{
	tx_queue_element_t * element;
	DWORD mresult;

	mresult = WaitForSingleObject(tx_queue.tx_queue_mutex, TX_MUTEX_TIMEOUT);
	if (mresult == WAIT_TIMEOUT || mresult == WAIT_FAILED) {
		printf("tx_dequeue(): failed to get tx mutex!!!\n");		
		return 1;
	}

	if(tx_queue.size == 0){
		//tx queue is empty
		ReleaseMutex(tx_queue.tx_queue_mutex);
		return 2;
	}

	//remove from the head of the queue
	element = tx_queue.head;
	tx_queue.head = element->previous;

	//now copy everything in to the user buffer
	memcpy(destination, &element->tx_data, sizeof(TX_DATA_STRUCT));

	free(element);

	tx_queue.size--;

	if(tx_queue.size == 0){
		tx_queue.head = NULL;
		tx_queue.tail = NULL;
	}

	ReleaseMutex(tx_queue.tx_queue_mutex);
	return 0;
}

//remove all elements from the queue
void empty_tx_queue()
{
	TX_DATA_STRUCT tx_data;

	while(tx_dequeue(&tx_data) == 0){
		;
	}
}


VOID 
GetApiStatistics(
	HANDLE drv, 
	PAPI_STATS pApi_stats
	)
{
	DWORD ln;

	DeviceIoControl(
		drv,
		Ioctl_Get_API_Stats,
		(LPVOID)pApi_stats,
		sizeof(API_STATS),
		(LPVOID)pApi_stats,
		sizeof(API_STATS),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		);
}
