/*
	Sangstrm.H - Streaming Device Driver (S508/S514) User Interface Header
	Copyright (c) 2001 Sangoma Technologies Inc.
	v2.00 September 2002

	Set tabs to 4 spaces when viewing.
*/

//
// DeviceIoControl command codes. Must include "winioctl.h" in app for
// definition of CTL_CODE and defines.
//

// Standard CBLOCK-passing command:
// Input: ptr to BSTRM_MAILBOX_STRUCT structure
// Output: ptr to BSTRM_MAILBOX_STRUCT structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
#define IoctlCodeCblockCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 10, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
//Structure used for transmission
//
typedef struct _TX_DATA_STRUCT{

	unsigned char  operation_status;
	unsigned short frame_length;
	unsigned char  frame_data[PRI_MAX_LENGTH_TX_DATA_BLOCK];

}TX_DATA_STRUCT;

//possible values for 'operation_status' on return
#define TX_OK				0
#define TX_TIMEOUT			1
#define TX_RETRY			3
#define TX_DATA_TOO_LONG	4
#define TX_DATA_TOO_SHORT	5

// Command to send a frame:
// Input: ptr to TX_DATA_STRUCT structure
// Output: ptr to TX_DATA_STRUCT structure (may be same as input)
// Uses Direct I/O (Irp->MdlAddress). Blocking call on device handle.
// Note: Frame only queued for transmission when 'operation_status' is set to TX_OK.
//		 If not, must repeat this command with reinitialized structure.
#define IoctlCodeWriteCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 11, METHOD_OUT_DIRECT, FILE_ANY_ACCESS)

// Command to read a frame:
// Input: NULL
// Output: ptr to STREAMING_READ_INFO (plus adequate data storage area)
// Uses Direct I/O (Irp->MdlAddress). Blocking call on device handle.
// Note: Since this command blocks all further I/O requests, it must be done
//       on a separate device handle. Just open the same device again and use
//       this handle for reading.
#define IoctlCodeReadCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 12, METHOD_IN_DIRECT, FILE_ANY_ACCESS)

// Command to get devices available for use.
// This command is used only by Configurator.exe.
// Input: NULL
// Output: ptr to SANGSTRM_DEVICE structure
// Uses Buffered I/O. Synchronous call. Used by
// configurator program only.
#define IoctlGetDevicesCommand	CTL_CODE(FILE_DEVICE_UNKNOWN, 14, METHOD_BUFFERED, FILE_ANY_ACCESS)

// Command to set T1/E1 configuration on S5144/S5147 cards.
// The default configuration (set on driver start up) is:
//
// media	= WANOPT_MEDIA_T1;
// lcode	= WANOPT_LC_B8ZS;
// frame	= WANOPT_FR_ESF;
// lbo		= WANOPT_T1_LBO_0_DB;
// te_clock	= WANOPT_NORMAL_CLK;
// active_ch= TE_ENABLE_ALL_CHANNELS;
//
// Input: ptr to TE_CFG_T structure
// Output: ptr to TE_CFG_T structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
// On return check the return_code member.
#define IoctlSetTEConfiguration CTL_CODE(FILE_DEVICE_UNKNOWN, 15, METHOD_BUFFERED, FILE_ANY_ACCESS)

// 
// Undocumented.
#define Ioctl_TE_56K_MonitorCommand CTL_CODE(FILE_DEVICE_UNKNOWN, 16, METHOD_BUFFERED, FILE_ANY_ACCESS)

////////////////////////////////////////////////////////
// Commands used with Ioctl_TE_56K_MonitorCommand

#define FT1_READ_STATUS		0x54	//for the FT1 card only
#define TE1_56K_STAT        0x62
#define GET_MEDIA_TYPE		0x63
#define FLUSH_TE1_PMON		0x64
#define READ_REGISTER		0x65

// End of Commands used with Ioctl_TE_56K_MonitorCommand
////////////////////////////////////////////////////////

// 
// Valid *only* for TE1 card (S5144/S5147).
//
// Input : ptr to T1_E1_SIGNALING_STATUS_STRUCT structure
// Output: ptr to T1_E1_SIGNALING_STATUS_STRUCT structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
#define Ioctl_GET_Rx_ABCD_Bits CTL_CODE(FILE_DEVICE_UNKNOWN, 17, METHOD_BUFFERED, FILE_ANY_ACCESS)


// Internal use.
// Undocumented.
#define Ioctl_GET_MEM_DUMP CTL_CODE(FILE_DEVICE_UNKNOWN, 18, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Input : ptr to API_STATS structure
// Output: ptr to API_STATS structure (may be same as input)
// Uses Buffered I/O. Synchronous call.
#define Ioctl_Get_API_Stats CTL_CODE(FILE_DEVICE_UNKNOWN, 19, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* return codes from IoctlSetTEConfiguration */
#define COMMAND_PERFORMED_SUCCESSFULLY      0x00		
#define DO_SET_BSTRM_CONFIGURATION      	0x02 /* SET_BSTRM_CONFIGURATION or SET_ASY_CONFIGURATION must be done
													before using this command */ 
#define DO_DISABLE_BSTRM_COMMUNICATIONS		0x03 /* use DISABLE_BSTRM_COMMUNICATIONS or DISABLE_ASY_COMMUNICATIONS
													before reconfiguring the driver */
#define INVALID_TOTAL_RX_BUFFER_SIZE		0x04 /* SIZEOF_RX_BFR_LGTH_IND(4 bytes) + 
													BSTRM_DATA_RX_STATUS_EL_STRUCT(16 bytes)+
												    max_BSTRM_data_field_length(set with SET_BSTRM_CONFIGURATION command)+
												    NUMBER_OF_CRC_BYTES(2 bytes) won't fit in to total RX buffer size, 
													requested by the user, devided by NO_DRVR_RX_BFRS (4) .
												 */
#define TE_CONFIGURATION_FAILED				0x06 /* Failed to set T1/E1 configuration. 
													 Check values passed to IoctlSetTEConfiguration command.
													 Make sure the card type is S5144 or S5147.*/

//
// Driver-defined return codes for CBLOCK-associated commands.
//
#define Z80_TIMEOUT_ERROR		0xAA	//firmware not responding to command
#define CARD_RELOADING_FAILED	0xAB	//look at bitstrmlog.txt file in \WINNT\System32\drivers

//
// Possible values for Status member of STREAMING_READ_INFO structure
// when IoctlCodeReadCommand returns.
//
#define RX_DATA_AVAILABLE	0x01	//Data was read.
#define RX_DATA_TIMEOUT		0x03	//Notification that no data was received in the
									//last 2 seconds.

//
// Equivalent to Receive Status Element in Manual. Must be packed on 1-byte boundary.
// Data of variable length follows.
//
#pragma pack(1)

typedef struct _STREAMING_READ_INFO {
	UCHAR Status;					// set to 0x01 on return if frame read
	USHORT FrameLength;				// set to length of frame that follows
	UCHAR MiscBits;					// various frame-related bits
	USHORT TimeStamp;				// timestamp of frame in milliseconds
	UCHAR Reserved[10];
	UCHAR Data[1];					// variable-length frame data
} STREAMING_READ_INFO, *PSTREAMING_READ_INFO;

typedef struct{
	USHORT HWareType;//CardType;
	
	ULONG  BusNumber;
	ULONG  SlotNumber;
	USHORT CPU;
	USHORT comms_port;
	
	ULONG  Memory;
	ULONG  IRQ;
	ULONG  IO_Port;

}SANGSTRM_DEVICE;

#pragma pack()

//////////////////////////////////////////////////////

#define MB_SIZE					16	//size of mail box without the data area
#define MAILBOX_DATA		    16	//offset of data area in the mail box structure
#define MAILBOX_RCODE		    4
//#define MAILBOX_BUFFER_LENGTH	2
//#define MAILBOX_COMMAND			1
     
#define TIMEOUT_INTERVAL			20000

// SDLA adapter port constants
//
//the offset from the card I/O port base address of the adapter
//segment and PC window select latch 
#define OFF_PC_ADDR_WS_SELECT 	0x01

// the offset from the card I/O port base address
// of the Z80 window select latch
#define OFF_Z80_WIN_SELECT 	0x02	

#define STANDARD_WINDOW     0x07

// card 64K segment address settings
// bits 4-5 of configuration byte written to port (base address + 1)
#define SEG_D	0x08
#define SEG_A   0x00
#define SEG_C   0x10
#define SEG_E   0x18

typedef struct 
{
	USHORT offset;
    UCHAR window;
} woffptr;

//Memory dump definitions
#define HEX_BASE				16				// 
#define MEMORY_DEF_ADDR			"00000"			// default start memory addr
#define MEMORY_DEF_LEN			"100"			// default dump memory length
#define MAX_SIZEOF_S508_MEMORY	(128*1024)

//Memory Dump sturcture
typedef struct 
{
	UCHAR	return_code;
	ULONG	offset;
	USHORT	length;
	UCHAR	data[2000];
} MEMORY_DUMP, *PMEMORY_DUMP;

typedef struct 
{
	UCHAR	return_code;
	ULONG	time_fill_char_tx_counter;
} API_STATS, *PAPI_STATS;

//////////////////////////////////////////////////////

//
//T1/E1 related declarations
//
//
// Possible values for members of TE_CFG_T structure:
//
/* Possible values for 'media' member */
/* The media (connection) type */
#define WANOPT_MEDIA_NONE		0x00	/* no DSU/CSU or old T1 connection */
#define WANOPT_MEDIA_T1			0x01	/* T1 connection */
#define WANOPT_MEDIA_E1			0x02	/* E1 connection */
#define WANOPT_MEDIA_56K		0x03	/* 56K connection */

/* End of values for 'media' member */
/*==================================*/

/* Possible values for 'lcode' member */
/* The line code */
#define WANOPT_LC_AMI           0x01	/* for T1 and E1 */
#define WANOPT_LC_B8ZS          0x02	/* for T1 only */
#define WANOPT_LC_HDB3          0x03	/* for E1 only */
/* End of values for 'lcode' member */
/*==================================*/

/* Possible values for 'frame' member */
/* The Frame mode (T1) */
#define WANOPT_FR_ESF           0x01
#define WANOPT_FR_D4            0x02
#define WANOPT_FR_ESF_JAPAN     0x03

/* The frame mode (E1) */
#define WANOPT_FR_CRC4          0x04
#define WANOPT_FR_NCRC4         0x05
/* End of values for 'frame' member */
/*==================================*/

/* Possible values for 'lbo' member */
/* Line Build Out */
/* For T1 only */
#define WANOPT_T1_LBO_0_DB      0x01
#define WANOPT_T1_LBO_75_DB     0x02
#define WANOPT_T1_LBO_15_DB     0x03
#define WANOPT_T1_LBO_225_DB    0x04
#define WANOPT_T1_0_110         0x05
#define WANOPT_T1_110_220       0x06
#define WANOPT_T1_220_330       0x07
#define WANOPT_T1_330_440       0x08
#define WANOPT_T1_440_550       0x09
#define WANOPT_T1_550_660       0x0A

/* For E1 only */
#define WANOPT_E1_120           0x0B
#define WANOPT_E1_75            0x0C
/* End of values for 'lbo' member */
/*==================================*/

/* Possible values for 'te_clock' member */
/* Clocking Master/Normal */
#define WANOPT_NORMAL_CLK	0x01
#define WANOPT_MASTER_CLK	0x02
/* End of values for 'te_clock' member */
/*=====================================*/

/* Possible values for 'active_ch' member */
/* Active Channels (active_ch) */
#define TE_ENABLE_ALL_CHANNELS	0x7FFFFFFF

/* To enable selected channels set bits in 'active_ch'.
   For example: if bit 0 is set, channel 1 is enabled,
   if bit 0 is reset, channel 1 is disabled.*/
/* End of values for 'active_ch' member */
/*=====================================*/

typedef struct TE_CFG {
	unsigned char media;
	unsigned char lcode;
	unsigned char frame;
	unsigned char lbo;
	unsigned char te_clock;
	unsigned long active_ch;
	unsigned char return_code;
} TE_CFG_T;

/* TE1 Alram bit mask */
#define BIT_ALOS_ALARM		0x0001
#define BIT_LOS_ALARM		0x0002
#define BIT_ALTLOS_ALARM	0x0004
#define BIT_OOF_ALARM		0x0008
#define BIT_RED_ALARM		0x0010
#define BIT_AIS_ALARM		0x0020
#define BIT_OOSMF_ALARM		0x0040
#define BIT_OOCMF_ALARM		0x0080
#define BIT_OOOF_ALARM		0x0100
#define BIT_RAI_ALARM		0x0200
#define BIT_YEL_ALARM		0x0400
#define BIT_TE1_ALARM		0x8000	/* for Windows only */

/* Performamce monitor counter defines */
#define frm_bit_error		pmon1	/* E1/T1   */
#define oof_errors			pmon2	/* T1 only */
#define far_end_blk_errors	pmon2	/* E1 only */
#define bit_errors			pmon3	/* T1 only */
#define crc_errors			pmon3	/* E1 only */
#define lcv					pmon4	/* E1/T1   */

#define ALOS_ALARM(val)		(val & BIT_ALOS_ALARM) ? "ON" : "OFF"
#define LOS_ALARM(val)		(val & BIT_LOS_ALARM) ? "ON" : "OFF"
#define ALTLOS_ALARM(val)	(val & BIT_ALTLOS_ALARM) ? "ON" : "OFF"
#define OOF_ALARM(val)		(val & BIT_OOF_ALARM) ? "ON" : "OFF"
#define RED_ALARM(val)		(val & BIT_RED_ALARM) ? "ON" : "OFF"
#define AIS_ALARM(val)		(val & BIT_AIS_ALARM) ? "ON" : "OFF"
#define OOSMF_ALARM(val)	(val & BIT_OOSMF_ALARM) ? "ON" : "OFF"
#define OOCMF_ALARM(val)	(val & BIT_OOCMF_ALARM) ? "ON" : "OFF"
#define OOOF_ALARM(val)		(val & BIT_OOOF_ALARM) ? "ON" : "OFF"
#define RAI_ALARM(val)		(val & BIT_RAI_ALARM) ? "ON" : "OFF"
#define YEL_ALARM(val)		(val & BIT_YEL_ALARM) ? "ON" : "OFF"

/* 56K Alram macros */
#define RRA(alarm) ((alarm>>8)&0xFF)
#define RRC(alarm) ((alarm)&0xFF)

#define INS_ALARM_56K(alarm) ((RRA(alarm)&0x0F) > 8) ? "GREEN" : "RED"
#define DMI_ALARM_56K(alarm) (RRC(alarm)&0x80) ? "RED" : "OFF"
#define ZCS_ALARM_56K(alarm) (RRC(alarm)&0x40) ? "RED" : "OFF"
#define CMI_ALARM_56K(alarm) (RRC(alarm)&0x20) ? "RED" : "OFF"
#define OOS_ALARM_56K(alarm) (RRC(alarm)&0x10) ? "RED" : "OFF"
#define OOF_ALARM_56K(alarm) (RRC(alarm)&0x08) ? "RED" : "OFF"
#define DLP_ALARM_56K(alarm) (RRC(alarm)&0x04) ? "RED" : "OFF"
#define UMC_ALARM_56K(alarm) (RRC(alarm)&0x02) ? "RED" : "OFF"
#define RLOS_ALARM_56K(alarm) (RRA(alarm)&0x80) ? "RED" : "OFF"


//must be divisibe by 24 for T1 line!!
#define T1_TX_RX_LENGTH		24*30 //720
//must be divisibe by 32 for E1 line!!
#define E1_RX_LENGTH		32*32 //1024
//must be divisibe by 31 for E1 line!!
#define E1_TX_LENGTH		32*31 //992

